// Kingdom of Kod
// ==============
// -Adventures of the heroes in the 'Manga Guide to Databases'

// -Filename: rw_cli_args.hpp
// -Command-line parsing & help support functions

#pragma once

#include <cstdlib>

#include "rw_gen_utils.hpp"
#include "rw_user_args.hpp"

namespace rw {

/** This is the overall (and single) CLI11's 'App' object used for parsing our
 * own program's arguments */
static CLI::App cli_app{
    "Kingdom of Kod: Adventures of the heroes in the 'Manga Guide to "
    "Databases'\n"};

/** Flag to track if the help flag was passed in as an argument */
bool have_help{false};

/**-----------------------------------------------------------------------------
 * Configure for, and parse, any command-line arguments (the actual work itself)
 *
 * @param argc [I] the CLI arguments count
 * @param argv [I] the CLI arguments C-string array
 * @return success code == 0, or a CLI11-specific parse error code
 *
 * @note: this function is called from the parse_args() wrapper function
 *
 * @b ref'd: rw::parse_args()
 */
int handle_args(int argc, char** argv) {
  //

  // NOTE HACK:
  //=============
  // The point of this hack is: ''get to the throw CLI::CallForHelp();''.
  // IE., if we actually receive one, the 'have_help' flag will be set, and the
  // throw will trigger a clean program exit (since it's both a '0' status
  // return and 'have_help' is set), from within the calling rw::parse_args()
  // wrapper function. The CLI::CallForHelp() 'error' forces a print of the help
  // display first (via 'cli_app.exit(e);' inside the catch block), before the
  // program itself exits from within parse_args() following the throw.
  //
  // The whole thing is just a messy hack to deal with CLI11's (wrong) help
  // flag behavior; namely, continuing on with the program's execution even when
  // a help flag is passed in. It shouldn't.
  //
  // https://github.com/CLIUtils/CLI11/blob/master/examples/modhelp.cpp
  //
  // first, disable the in-built default help handler. this removes the
  // pre-defined, 'help' named-item object and detaches it from CLI11's internal
  // state. this condition is needed for the help-check pointer dereferencing
  // 'if (*help)' call made below, to bypass CLI11's default (wrong) behavior.
  cli_app.set_help_flag();
  // this just (re)defines a new custom flag for activating help, since disabled
  auto help = cli_app.add_flag("-h, --help", "Display this help and quit");

  try {
    cli_app.parse(argc, argv);  // the actual CLI parsing itself

    // special block just for the custom help handler
    if (*help) {                 // only valid if a call for help was made
      have_help = true;          // checked within parse_args()
      throw CLI::CallForHelp();  // caught immediately below, sets a '0' status
    }

  } catch (const CLI::ParseError& e) {
    return cli_app.exit(e);
  }

  return 0;
}

/**-----------------------------------------------------------------------------
 * Parse any command-line arguments
 *
 * @param argc [I] the CLI arguments count
 * @param argv [I] the CLI arguments C-string array
 *
 * This is a wrapper that ensures correct program exit on either argument
 * errors or the help flag, and calls to here are passed on to the actual
 * handling function.
 *
 * @note: this function calls the handle_args() function
 *
 * @b ref'd: rw::handle_args(int argc, char** argv)
 */
void parse_args(int argc, char** argv) {
  if (auto bad_arg = handle_args(argc, argv))  // non-zero ret == error state
    std::exit(bad_arg);
  else if (have_help)  // simple hack to prevent program execution on --help
    std::exit(0);
}

}  // namespace rw

// Copyright (2021)
// License MIT (Expat) https://opensource.org/licenses/MIT
